package com.framsticks.communication;

import java.util.List;

import com.framsticks.communication.queries.RegistrationRequest;
import com.framsticks.util.dispatching.AtOnceDispatcher;
import com.framsticks.util.dispatching.Dispatcher;
import com.framsticks.util.dispatching.Dispatching;
import com.framsticks.util.dispatching.RunAt;
import com.framsticks.util.dispatching.ThrowExceptionHandler;

import org.apache.log4j.Logger;


/**
 * @author Piotr Sniegowski
 */
public class Subscription<C> {

	private final static Logger log = Logger.getLogger(Subscription.class);

	private final ClientSideManagedConnection connection;
	private final String path;
	private final String registeredPath;
	private final Dispatcher<C> dispatcher;

	private EventCallback eventCallback;

	public Subscription(ClientSideManagedConnection connection, String path, String registeredPath, Dispatcher<C> dispatcher) {
		this.connection = connection;
		this.path = path;
		this.registeredPath = registeredPath;
		this.dispatcher = dispatcher;
	}

	public String getPath () {
		return path;
	}

	public String getRegisteredPath() {
		return registeredPath;
	}

	@Override
	public String toString() {
		return path + "(" + registeredPath + ")";
	}

	public void unsubscribe(final ClientSideResponseFuture future) {
		//@todo remove that /cli/ prefix, when registeredPath will be a fully qualified path
		connection.send(new RegistrationRequest().register(false).path(registeredPath), AtOnceDispatcher.getInstance(), new ClientSideResponseFuture(future) {

			@Override
			protected void processOk(Response response) {
				assert response.hasFiles();
				log.debug("unsunscribed " + this);
				future.pass(response);
			}
		});
	}

	public EventCallback getEventCallback() {
		return eventCallback;
	}

	public Dispatcher<C> getDispatcher() {
		return dispatcher;
	}

	public void setEventCallback(EventCallback eventCallback) {
		this.eventCallback = eventCallback;
	}

	public void dispatchCall(final List<File> files) {
		Dispatching.dispatchIfNotActive(dispatcher, new RunAt<C>(ThrowExceptionHandler.getInstance()) {
			@Override
			protected void runAt() {
				eventCallback.call(files);
			}
		});
	}
}
