package com.framsticks.core;

import javax.annotation.Nonnull;

import com.framsticks.params.AccessInterface;
import com.framsticks.params.CompositeParam;
import com.framsticks.params.FramsClass;
import com.framsticks.params.PrimitiveParam;
import com.framsticks.params.Registry;
import com.framsticks.params.ValueParam;
import com.framsticks.params.types.ProcedureParam;
import com.framsticks.util.dispatching.Dispatcher;
import com.framsticks.util.dispatching.ExceptionResultHandler;
import com.framsticks.util.dispatching.Future;
import com.framsticks.util.dispatching.Joinable;
import com.framsticks.util.dispatching.JoinableDispatcher;

public interface Tree extends Dispatcher<Tree>, Joinable, ExceptionResultHandler {

	public @Nonnull Node getAssignedRoot();
	public void assignRootParam(CompositeParam param);
	public void assignRootObject(Object object);

	public @Nonnull AccessInterface prepareAccess(CompositeParam param);
	public void takeAllFrom(Registry source);

	public FramsClass getInfoFromCache(String id);

	public void putInfoIntoCache(FramsClass framclass);

	/**
	 *
	 * Functions accepts ValueParam, because it is also possible to get number of List elements.
	 *
	 */
	public void get(Path path, ValueParam param, Mode mode, Future<Object> future);

	public void get(Path path, Mode mode, Future<Path> future);

	public void call(Path path, ProcedureParam param, Object[] arguments, Future<Object> future);

	/**
	 *
	 * Functions accepts PrimitiveParam<?>, because it is not possible to set number of List elements.
	 *
	 */
	public void set(Path path, PrimitiveParam<?> param, Object value, Future<Integer> future);

	public void info(Path path, Future<FramsClass> future);

	public void setExceptionHandler(ExceptionResultHandler handler);

	public ExceptionResultHandler getExceptionHandler();

	public void setDispatcher(JoinableDispatcher<Tree> dispatcher);

	public JoinableDispatcher<Tree> getDispatcher();

}
