package com.framsticks.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import com.framsticks.util.dispatching.AbstractJoinable;
import com.framsticks.util.dispatching.Dispatcher;
import com.framsticks.util.dispatching.JoinableDispatcher;
import com.framsticks.util.dispatching.Task;
import com.framsticks.util.dispatching.ThrowExceptionHandler;

import javax.swing.*;
import com.framsticks.util.dispatching.RunAt;

/**
 * @author Piotr Sniegowski
 */
public class SwingDispatcher<C> extends AbstractJoinable implements JoinableDispatcher<C> {

	@SuppressWarnings("rawtypes")
	protected static final SwingDispatcher instance = new SwingDispatcher();

	@SuppressWarnings("unchecked")
	public static <C> Dispatcher<C> getInstance() {
		return (Dispatcher<C>) instance;
	}

	public SwingDispatcher() {
		dispatch(new RunAt<C>(ThrowExceptionHandler.getInstance()) {
			@Override
			protected void runAt() {
				Thread.currentThread().setName("gui");
			}
		});
	}

	@Override
	public final boolean isActive() {
		return SwingUtilities.isEventDispatchThread();
	}

	@Override
	public final void dispatch(RunAt<? extends C> runnable) {
		if (runnable instanceof Task) {
			final Task<?> task = (Task<?>) runnable;
			Timer timer = new Timer(0, null);
			timer.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent event) {
					task.run();
				}

			});
			timer.setInitialDelay((int) (task.getMoment() - System.currentTimeMillis()));
			timer.setRepeats(false);
			timer.start();
			return;
		}
		SwingUtilities.invokeLater(runnable);
	}

	@Override
	public String getName() {
		return "gui";
	}

	@Override
	protected void joinableStart() {

	}

	@Override
	protected void joinableInterrupt() {
		finish();
	}

	@Override
	protected void joinableFinish() {

	}

	@Override
	protected void joinableJoin() throws InterruptedException {

	}

}
