package com.framsticks.gui.controls;

import com.framsticks.core.Tree;
import com.framsticks.core.Path;
import com.framsticks.gui.Frame;
import com.framsticks.gui.Gui;
import com.framsticks.params.Param;
import com.framsticks.params.ValueParam;
import com.framsticks.params.types.ProcedureParam;
import com.framsticks.util.dispatching.ExceptionResultHandler;
import com.framsticks.util.dispatching.FutureHandler;
import com.framsticks.util.dispatching.RunAt;
import com.framsticks.util.dispatching.ThrowExceptionHandler;

import javax.swing.*;
import javax.swing.border.BevelBorder;
import javax.swing.tree.TreePath;

import org.apache.log4j.Logger;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.IdentityHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

@SuppressWarnings("serial")
public class ProcedureControl extends Control implements ControlOwner {

	private static final Logger log = Logger.getLogger(ProcedureControl.class);

	protected final JButton procedureButton;

	final protected Map<ValueParam, ValueControl> components = new IdentityHashMap<>();

	public ProcedureControl(ProcedureParam procedureParam) {
		super(procedureParam);

		procedureButton = new JButton("Call");
		procedureButton.setName("call");

		this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));

		Gui.fillWithControls(this, procedureParam.getArgumentsType(), components, ValueControl.class);

		if (components.size() != procedureParam.getArgumentsType().size()) {
			procedureButton.setEnabled(false);
		}
		if (!components.isEmpty()) {
			this.setBorder(new BevelBorder(BevelBorder.RAISED));
		}

		procedureButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {

				final Path path = getFrame().getTreeModel().convertToPath(getCurrentTreePath());

				final List<Object> arguments = new LinkedList<Object>();
				for (Param arg : getParam().getArgumentsType()) {
					Object value = components.get(arg).getCurrentValue();
					arguments.add(value);
					log.debug("argument " + arg + ": " + value);
				}
				//TODO FEH: make it show dialog
				final ExceptionResultHandler handler = ThrowExceptionHandler.getInstance();

				path.getTree().dispatch(new RunAt<Tree>(handler) {
					@Override
					protected void runAt() {
						path.getTree().call(path, getParam(), arguments.toArray(), new FutureHandler<Object>(handler) {

							@Override
							public void result(Object result) {

							}
						});
					}
				});
			}
		});
		this.add(procedureButton);

	}

	@Override
	public JPanel getPanel() {
		return this;
	}

	@Override
	public ProcedureParam getParam() {
		return (ProcedureParam) param;
	}

	@Override
	protected void updateEnabled(boolean enabled) {
		procedureButton.setEnabled(enabled);
		for (ValueControl vc : components.values()) {
			vc.setUserEnabled(enabled);
		}
	}

	@Override
	public Frame getFrame() {
		return owner.getFrame();
	}

	@Override
	public TreePath getCurrentTreePath() {
		return owner.getCurrentTreePath();
	}

}
